/*!
 * Copyright (c) Metaways Infosystems GmbH, 2011
 * LGPLv3, http://www.arcavias.com/en/license
 */


Ext.ns('MShop.panel.plugin');

MShop.panel.plugin.ItemUi = Ext.extend(MShop.panel.AbstractListItemUi, {
	initComponent : function() {
		
		this.items = [ {
			xtype : 'tabpanel',
			activeTab : 0,
			border : false,
			itemId : 'MShop.panel.plugin.ItemUi',
			plugins : [ 'ux.itemregistry' ],
			items : [ {
				xtype : 'panel',
				title : MShop.I18n.dt( 'client/extjs', 'Basic' ),
				border : false,
				layout : 'hbox',
				layoutConfig : {
					align : 'stretch'
				},
				itemId : 'MShop.panel.plugin.ItemUi.BasicPanel',
				plugins : [ 'ux.itemregistry' ],
				defaults : {
					bodyCssClass : this.readOnlyClass
				},
				items : [ {
					xtype : 'form',
					title : MShop.I18n.dt( 'client/extjs', 'Details' ),
					flex : 1,
					ref : '../../mainForm',
					autoScroll : true,
					items : [ {
						xtype : 'fieldset',
						style: 'padding-right: 25px;',
						border : false,
						labelAlign : 'top',
						defaults: {
							anchor : '100%'
						},
						items : [ {
							xtype : 'displayfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs',  'ID' ),
							name : 'plugin.id'
						}, {
							xtype : 'MShop.elements.status.combo',
							name : 'plugin.status'
						}, {
							xtype : 'combo',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Type' ),
							name : 'plugin.typeid',
							mode : 'local',
							store : MShop.GlobalStoreMgr.get('Plugin_Type'),
							displayField : 'plugin.type.label',
							valueField : 'plugin.type.id',
							forceSelection : true,
							triggerAction : 'all',
							allowBlank : false,
							typeAhead : true,
							listeners: {
								'render' : {
									fn: function() {
										var record, index = this.store.find( 'plugin.type.code', 'order' );
										if( ( record = this.store.getAt( index ) ) ) {
											this.setValue( record.id );
										}
									}
								}
							}
						}, {
							xtype : 'textfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Provider' ),
							name : 'plugin.provider',
							allowBlank : false,
							maxLength : 255,
							emptyText : MShop.I18n.dt( 'client/extjs', 'Name of the plugin provider class (required)' )
						}, {
							xtype : 'textfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Label' ),
							name : 'plugin.label',
							allowBlank : false,
							maxLength : 255,
							emptyText : MShop.I18n.dt( 'client/extjs', 'Internal name (required)' )
						}, {
							xtype : 'numberfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Position' ),
							name : 'plugin.position',
							allowDecimals : false,
							allowBlank : false,
							value : 0
						}, {
							xtype : 'displayfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Created' ),
							name : 'plugin.ctime'
						}, {
							xtype : 'displayfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Last modified' ),
							name : 'plugin.mtime'
						}, {
							xtype : 'displayfield',
							fieldLabel : MShop.I18n.dt( 'client/extjs', 'Editor' ),
							name : 'plugin.editor'
						} ]
					} ]
				}, {
					xtype: 'MShop.panel.plugin.configui',
					layout: 'fit',
					flex: 1,
					data: ( this.record ? this.record.get('plugin.config') : {} )
				}]
			} ]
		} ];

		this.store.on('beforesave', this.onBeforeSave, this);

		MShop.panel.plugin.ItemUi.superclass.initComponent.call(this);
	},


	afterRender : function()
	{
		var label = this.record ? this.record.data['plugin.label'] : MShop.I18n.dt( 'client/extjs', 'new' );
		//#: Plugin item panel title with attribute label ({0}) and site code ({1)}
		var string = MShop.I18n.dt( 'client/extjs', 'Plugin: {0} ({1})' );
		this.setTitle( String.format( string, label, MShop.config.site["locale.site.label"] ) );

		MShop.panel.product.ItemUi.superclass.afterRender.apply( this, arguments );
	},


	onBeforeSave: function( store, data ) {

		var config = {};
		var editorGrid = this.findByType( 'MShop.panel.plugin.configui' );
		var first = editorGrid.shift();

		if( first ) {
			Ext.each( first.data, function( item, index ) {
				Ext.iterate( item, function( key, value, object ) {
					if( ( key = key.trim() ) !== '' ) {
						config[key] = (typeof value === "string") ? value.trim() : value;
					}
				}, this);
			});
		}

		if( data.create && data.create[0] ) {
			data.create[0].data['plugin.config'] = config;
		} else if( data.update && data.update[0] ) {
			data.update[0].data['plugin.config'] = config;
		}
	}
});

Ext.reg('MShop.panel.plugin.itemui', MShop.panel.plugin.ItemUi);